#define WIN32_LEAN_AND_MEAN
#include "windows.h"
#include "windowsx.h"

// === Structures Required For This Program ===================================

#pragma pack(1)                       //  Required for proper structure packing
typedef struct session_info_50 {
   LPSTR sesi50_cname;
   LPSTR sesi50_username;
   DWORD sesi50_key;
   USHORT sesi50_num_conns;
   USHORT sesi50_num_opens;
   DWORD sesi50_time;
   DWORD sesi50_idle_time;
   BYTE sesi50_protocol;
   BYTE pad1;
} SESSION_INFO_50, * PSESSION_INFO_50;

typedef struct _SESSION_INFO_1 {
    LPTSTR    sesi1_cname;              // client name (no backslashes)
    LPTSTR    sesi1_username;
    DWORD     sesi1_num_opens;
    DWORD     sesi1_time;
    DWORD     sesi1_idle_time;
    DWORD     sesi1_user_flags;
} SESSION_INFO_1, *PSESSION_INFO_1, *LPSESSION_INFO_1;

// === Function Prototypes ====================================================

BOOL WINAPI MainDlgProc( HWND, UINT, WPARAM, LPARAM );
BOOL GetSessionInfo( HWND );
BOOL W95GetSessionInfo( HWND );
BOOL WNTGetSessionInfo( HWND );

// === Global Variables =======================================================

HINSTANCE hInst;

// === Application Entry Point ================================================

int APIENTRY WinMain( HINSTANCE hInstance, HINSTANCE hPrev, LPSTR lpCmd,
                      int nShow )
{
   hInst = hInstance;
   DialogBox( hInstance, MAKEINTRESOURCE( 10000 ), NULL, MainDlgProc );
   return( FALSE );
}

// === Main Window (Dialog box) Proc ==========================================

BOOL WINAPI MainDlgProc( HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam )
{
   switch( msg )
   {
      case WM_INITDIALOG:
         GetSessionInfo( hDlg );
         return( TRUE );

      case WM_COMMAND:
         if( wParam == IDCANCEL )
         {
            EndDialog( hDlg, TRUE );
            return( TRUE );
         }
         if( wParam == IDOK )
         {
            SendDlgItemMessage( hDlg, 100, LB_RESETCONTENT, 0, 0 );
            GetSessionInfo( hDlg );
         }
         break;
   }
   return( FALSE );
}

// === App Callable Function ==================================================

BOOL GetSessionInfo( HWND hDlg )
{
   OSVERSIONINFO osvi;

   memset( &osvi, 0, sizeof(OSVERSIONINFO) );        //  Init version structure
   osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);     //  add structure size

   GetVersionEx( &osvi );                            //  Get Win32 version info
   if( osvi.dwPlatformId == VER_PLATFORM_WIN32_WINDOWS )       //  Is it Win95?
      return( W95GetSessionInfo( hDlg ) );                  //  If so, run this
   else
      return( WNTGetSessionInfo( hDlg ) );

   return( FALSE );
}                                                   //  End of GetSessionInfo()

// === Windows 95 Session Info Function =======================================

BOOL W95GetSessionInfo( HWND hDlg )
{
   PSESSION_INFO_50 psi50;
   LPBYTE bBuffer;
   DWORD dwStatus;
   USHORT usRead, usTotal;
   HANDLE hModule;
   char szStr[ 256 ];
   int i;
   DWORD ( CALLBACK *pNetSessionEnum )( LPSTR, short, LPSTR, USHORT,
                                        USHORT *, USHORT * );


   bBuffer = GlobalAllocPtr( GHND, 16384 );               //  Allocate a buffer
   if( ! bBuffer )
      return( FALSE );

   psi50 = (PSESSION_INFO_50)bBuffer;               //  Assign struct to buffer
   memset( psi50, 0, 16384 );                         //  Clear buffer contents
   hModule = LoadLibrary( "SVRAPI.DLL" );            //  Load Win95 net library
   if( hModule )
   {                                           //  Get address of enum function
      (FARPROC)pNetSessionEnum = GetProcAddress( hModule,
                                                 "NetSessionEnum" );
      if( pNetSessionEnum )                  //  If function was found, call it
      {
         usRead = usTotal = 0;          //  Initialize function variables first
         dwStatus = pNetSessionEnum( NULL, 50, (LPSTR)psi50,     //  Win95 uses
                                     16384, &usRead, &usTotal );   //  level 50
         if( ! dwStatus && usRead )   //  If no error and something was read...
         {
            for( i = 0; i < (int)usRead; i++ )       //  Loop through all items
            {
               wsprintf( szStr, "%s on %s",       //  Create string from pieces
                         psi50->sesi50_username,               //  Display user
                         psi50->sesi50_cname );          //  and computer names
               SendDlgItemMessage( hDlg, 100, LB_ADDSTRING, 0,
                                   (LPARAM)szStr );      //  And add to listbox
               ++psi50;                               //  Next structure please
            }
         }
      }
      FreeLibrary( hModule );             //  Free library if previously loaded
   }
   GlobalFreePtr( bBuffer );                 //  Free allocated buffer and exit
   return( TRUE );
}                                                //  End of W95GetSessionInfo()

// ============================================================================

BOOL WNTGetSessionInfo( HWND hDlg )
{
   HANDLE hModule;
   DWORD dwStatus, dwRead, dwTotal;
   int i;
   SESSION_INFO_1 *psi1, *opsi1;
   char szUsername[ 128 ], szCname[ 128 ], szOutput[ 256 ];
   DWORD ( CALLBACK *pNetSessionEnum )( LPWSTR, LPWSTR, LPWSTR, DWORD,
                                        LPBYTE *, DWORD, LPDWORD, LPDWORD,
                                        LPDWORD );

   hModule = LoadLibrary( "NETAPI32.DLL" );
   if( ! hModule )
      return( FALSE );

   (FARPROC)pNetSessionEnum = (FARPROC)GetProcAddress( hModule,
                                                       "NetSessionEnum" );
   if( ! pNetSessionEnum )
   {
      FreeLibrary( hModule );
      return( FALSE );
   }

   psi1 = (SESSION_INFO_1 *)GlobalAllocPtr( GHND, 
                                            sizeof(SESSION_INFO_1) * 64 );
   if( ! psi1 )
   {
      FreeLibrary( hModule );
      return( FALSE );
   }
   opsi1 = psi1;

   dwStatus = pNetSessionEnum( NULL, NULL, NULL, 1, 
                               (LPBYTE *)&psi1, sizeof(SESSION_INFO_1) * 64,
                               &dwRead, &dwTotal, NULL );

   if( ! dwStatus )
   {
      for( i = 0; i < (int)dwRead; i++ )
      {
         WideCharToMultiByte( CP_ACP, 0, 
                              (LPWSTR)psi1->sesi1_username, -1,
                              szUsername, 128, NULL, NULL );

         WideCharToMultiByte( CP_ACP, 0,
                              (LPWSTR)psi1->sesi1_cname, -1,
                              szCname, 128, NULL, NULL );

         wsprintf( szOutput, "%s on %s", szUsername, szCname );
         SendDlgItemMessage( hDlg, 100, LB_ADDSTRING, 0, (LPARAM)szOutput );

         ++psi1;
      }
   }
   GlobalFreePtr( opsi1 );
   FreeLibrary( hModule );
   return( TRUE );
}



